@@ -15,6 +15,7 @@ from .pythreejs import (
     Picker,
     Geometry,
     PlainGeometry,
+    PlainBufferGeometry,
     SphereGeometry,
     CylinderGeometry,
     BoxGeometry,
@@ -84,6 +85,7 @@ __all__ = ["Texture",
            "Picker",
            "Geometry",
            "PlainGeometry",
+           "PlainBufferGeometry",
            "SphereGeometry",
            "CylinderGeometry",
            "BoxGeometry",
@@ -143,9 +145,3 @@ def _jupyter_nbextension_paths():
         'dest': npm_pkg_name,
         'require': npm_pkg_name + '/extension'
     }]
-
-def _jupyter_labextension_paths():
-    return [{
-        'name': npm_pkg_name,
-        'src': 'staticlab'
-    }]
\ No newline at end of file
