@@ -16,7 +16,17 @@ from ipywidgets import Widget, DOMWidget, widget_serialization, Color
 from traitlets import (Unicode, CInt, Instance, Enum, List, Tuple, Dict, Float,
                        CFloat, Bool)
 from ._package import npm_pkg_name
+from .enums import (
+        Equations, DestinationFactors, SourceFactors, Side, Shading, Colors,
+        BlendingMode, Operations, MappingModes, WrappingModes, Filters,
+        DataTypes, PixelTypes, PixelFormats, CompressedTextureFormats,
+        Lines, Renderers)
+
 from math import pi, sqrt
+from .traits_numpy import array_serialization, shape_constraints
+from traittypes import Array
+import numpy as np
+
 
 def vector3(trait_type=CFloat, default=None, **kwargs):
     if default is None:
@@ -28,12 +38,15 @@ def vector2(trait_type=CFloat, default=None, **kwargs):
         default = [0, 0]
     return List(trait_type, default_value=default, minlen=2, maxlen=2, **kwargs)
 
+PYTHREEJS_VERSION='0.3.0-alpha.0'
 
 class Texture(Widget):
     _view_module = Unicode(npm_pkg_name).tag(sync=True)
     _model_module = Unicode(npm_pkg_name).tag(sync=True)
     _view_name = Unicode('TextureView').tag(sync=True)
     _model_name = Unicode('TextureModel').tag(sync=True)
+    _model_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
+    _view_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
 
 
 class ImageTexture(Texture):
@@ -56,25 +69,22 @@ class DataTexture(Texture):
     _model_name = Unicode('DataTextureModel').tag(sync=True)
 
     data = List(CInt).tag(sync=True)
-    format = Enum(['RGBAFormat', 'AlphaFormat', 'RGBFormat', 'LuminanceFormat',
-                   'LuminanceAlphaFormat'], 'RGBAFormat').tag(sync=True)
+    format = Enum(PixelFormats, 'RGBAFormat').tag(sync=True)
     width = CInt(256).tag(sync=True)
     height = CInt(256).tag(sync=True)
-    type = Enum(['UnsignedByteType', 'ByteType', 'ShortType',
-                 'UnsignedShortType', 'IntType', 'UnsignedIntType',
-                 'FloatType', 'UnsignedShort4444Type', 'UnsignedShort5551Type',
-                 'UnsignedShort565Type'], 'UnsignedByteType').tag(sync=True)
-    mapping = Enum(['UVMapping', 'CubeReflectionMapping',
-                    'CubeRefractionMapping', 'SphericalReflectionMapping',
-                    'SphericalRefractionMapping'], 'UVMapping').tag(sync=True)
-    wrapS = Enum(['ClampToEdgeWrapping', 'RepeatWrapping', 'MirroredRepeatWrapping'],
-                 'ClampToEdgeWrapping').tag(sync=True)
-    wrapT = Enum(['ClampToEdgeWrapping', 'RepeatWrapping', 'MirroredRepeatWrapping'],
-                 'ClampToEdgeWrapping').tag(sync=True)
-    magFilter = Enum(['LinearFilter', 'NearestFilter'], 'LinearFilter').tag(sync=True)
-    minFilter = Enum(['NearestFilter', 'NearestMipMapNearestFilter',
-                      'NearestMipMapLinearFilter', 'LinearFilter',
-                      'LinearMipMapNearestFilter'], 'NearestFilter').tag(sync=True)
+    type = Enum(DataTypes, 'UnsignedByteType').tag(sync=True)
+
+    # TODO: this enum includes both DataTypes and PixelTypes?
+    # type = Enum(['UnsignedByteType', 'ByteType', 'ShortType',
+    #              'UnsignedShortType', 'IntType', 'UnsignedIntType',
+    #              'FloatType', 'UnsignedShort4444Type', 'UnsignedShort5551Type',
+    #              'UnsignedShort565Type'], 'UnsignedByteType').tag(sync=True)
+
+    mapping = Enum(MappingModes, 'UVMapping').tag(sync=True)
+    wrapS = Enum(WrappingModes, 'ClampToEdgeWrapping').tag(sync=True)
+    wrapT = Enum(WrappingModes, 'ClampToEdgeWrapping').tag(sync=True)
+    magFilter = Enum(Filters, 'LinearFilter').tag(sync=True)
+    minFilter = Enum(Filters, 'NearestFilter').tag(sync=True)
     anisotropy = CInt(1).tag(sync=True)
 
 
@@ -97,6 +107,8 @@ class Object3d(Widget):
     _model_module = Unicode(npm_pkg_name).tag(sync=True)
     _view_name = Unicode('Object3dView').tag(sync=True)
     _model_name = Unicode('Object3dModel').tag(sync=True)
+    _model_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
+    _view_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
 
     position = vector3(CFloat).tag(sync=True)
     quaternion = List(CFloat).tag(sync=True)  # [x,y,z,w]
@@ -210,6 +222,8 @@ class Controls(Widget):
     _model_module = Unicode(npm_pkg_name).tag(sync=True)
     _view_name = Unicode('ControlsView').tag(sync=True)
     _model_name = Unicode('ControlsModel').tag(sync=True)
+    _model_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
+    _view_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
 
     controlling = Instance(Object3d, allow_none=True).tag(sync=True, **widget_serialization)
 
@@ -262,21 +276,29 @@ class Geometry(Widget):
     _model_module = Unicode(npm_pkg_name).tag(sync=True)
     _view_name = Unicode('GeometryView').tag(sync=True)
     _model_name = Unicode('GeometryModel').tag(sync=True)
+    _model_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
+    _view_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
 
 
 class PlainGeometry(Geometry):
     _view_name = Unicode('PlainGeometryView').tag(sync=True)
     _model_name = Unicode('PlainGeometryModel').tag(sync=True)
 
-    vertices = List(vector3(CFloat)).tag(sync=True)
-    colors = List(Color).tag(sync=True)
-    faces = List(List(CFloat)).tag(sync=True)
-    # TODO: type this better. Can handle lists of string colors, or lists of lists of string colors.
-    faceColors = Tuple().tag(sync=True)
-    # TODO: type this better. Can handle lists of vector3(CFloat), or lists of lists of vector3(CFloat).
-    faceNormals = Tuple().tag(sync=True)
-    # todo: faceVertexUvs = List(vector3(vector2(CFloat))).tag(sync=True)
+    vertices = Array(dtype='float32').tag(sync=True, **array_serialization).valid(shape_constraints(None,3))
+    faces = Array(dtype='uint32', default_value=np.empty(shape=(0,3), dtype='uint32')).tag(sync=True, **array_serialization).valid(shape_constraints(None,3))
 
+    # list of [[v1_r,v1_g,v1_b], [v2_r,v2_g,v2_b], [v3_r,v3_g,v3_b]] for each face
+    faceColors = Array(dtype='float32', default_value=np.empty(shape=(0,3,3), dtype='float32')).tag(sync=True, **array_serialization).valid(shape_constraints(None, 3, 3))
+    colors = List(Color).tag(sync=True)
+    faceNormals = Tuple().tag(sync=True)
+
+class PlainBufferGeometry(Geometry):
+    _view_name = Unicode('PlainBufferGeometryView').tag(sync=True)
+    _model_name = Unicode('PlainBufferGeometryModel').tag(sync=True)
+
+    vertices = Array(dtype='float32').tag(sync=True, **array_serialization).valid(shape_constraints(None,3))
+    faces = Array(dtype='uint32', default_value=np.empty(shape=(0,3), dtype='uint32')).tag(sync=True, **array_serialization).valid(shape_constraints(None,3))
+    colors = Array(dtype='float32', default_value=np.empty(shape=(0,3), dtype='float32'), help="Vertex colors").tag(sync=True, **array_serialization).valid(shape_constraints(None,3))
 
 class SphereGeometry(Geometry):
     _view_name = Unicode('SphereGeometryView').tag(sync=True)
@@ -461,23 +483,18 @@ class Material(Widget):
     _model_module = Unicode(npm_pkg_name).tag(sync=True)
     _view_name = Unicode('MaterialView').tag(sync=True)
     _model_name = Unicode('MaterialModel').tag(sync=True)
+    _model_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
+    _view_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
 
     # id = TODO
     name = Unicode(sync=True)
-    side = Enum(['FrontSide', 'BackSide', 'DoubleSide'], 'DoubleSide').tag(sync=True)
+    side = Enum(Side, 'DoubleSide').tag(sync=True)
     opacity = CFloat(1.0).tag(sync=True)
     transparent = Bool().tag(sync=True)
-    blending = Enum(['NoBlending', 'NormalBlending', 'AdditiveBlending',
-                     'SubtractiveBlending', 'MultiplyBlending',
-                     'CustomBlending'], 'NormalBlending').tag(sync=True)
-    blendSrc = Enum(['ZeroFactor', 'OneFactor', 'SrcColorFactor',
-                     'OneMinusSrcColorFactor', 'SrcAlphaFactor',
-                     'OneMinusSrcAlphaFactor', 'DstAlphaFactor',
-                     'OneMinusDstAlphaFactor'], 'SrcAlphaFactor').tag(sync=True)
-    blendDst = Enum(['DstColorFactor', 'OneMinusDstColorFactor',
-                     'SrcAlphaSaturateFactor'], 'OneMinusDstColorFactor').tag(sync=True)
-    blendEquation = Enum(['AddEquation', 'SubtractEquation',
-                          'ReverseSubtractEquation'], 'AddEquation').tag(sync=True)
+    blending = Enum(BlendingMode, 'NormalBlending').tag(sync=True)
+    blendSrc = Enum(DestinationFactors, 'SrcAlphaFactor').tag(sync=True)
+    blendDst = Enum(SourceFactors, 'OneMinusDstColorFactor').tag(sync=True)
+    blendEquation = Enum(Equations, 'AddEquation').tag(sync=True)
     depthTest = Bool(True).tag(sync=True)
     depthWrite = Bool(True).tag(sync=True)
     polygonOffset = Bool(True).tag(sync=True)
@@ -498,8 +515,8 @@ class BasicMaterial(Material):
     wireframeLinewidth = CFloat(1.0).tag(sync=True)
     wireframeLinecap = Unicode('round').tag(sync=True)
     wireframeLinejoin = Unicode('round').tag(sync=True)
-    shading = Enum(['SmoothShading', 'FlatShading', 'NoShading'], 'SmoothShading').tag(sync=True)
-    vertexColors = Enum(['NoColors', 'FaceColors', 'VertexColors'], 'NoColors').tag(sync=True)
+    shading = Enum(Shading, 'SmoothShading').tag(sync=True)
+    vertexColors = Enum(Colors, 'NoColors').tag(sync=True)
     fog = Bool().tag(sync=True)
     map = Instance(Texture, allow_none=True).tag(sync=True, **widget_serialization)
     lightMap = Instance(Texture, allow_none=True).tag(sync=True, **widget_serialization)
@@ -516,8 +533,7 @@ class LambertMaterial(BasicMaterial):
     emissive = Color('black').tag(sync=True)
     reflectivity = CFloat(1.0).tag(sync=True)
     refractionRatio = CFloat(0.98).tag(sync=True)
-    combine = Enum(['MultiplyOperation', 'MixOperation', 'AddOperation'],
-                   'MultiplyOperation').tag(sync=True)
+    combine = Enum(Operations, 'MultiplyOperation').tag(sync=True)
 
 
 class PhongMaterial(BasicMaterial):
@@ -529,8 +545,7 @@ class PhongMaterial(BasicMaterial):
     shininess = CFloat(30).tag(sync=True)
     reflectivity = CFloat(1.0).tag(sync=True)
     refractionRatio = CFloat(0.98).tag(sync=True)
-    combine = Enum(['MultiplyOperation', 'MixOperation', 'AddOperation'],
-                   'MultiplyOperation').tag(sync=True)
+    combine = Enum(Operations, 'MultiplyOperation').tag(sync=True)
 
 
 class DepthMaterial(Material):
@@ -555,7 +570,7 @@ class LineBasicMaterial(_LineMaterial):
     linecap = Unicode('round').tag(sync=True)
     linejoin = Unicode('round').tag(sync=True)
     fog = Bool().tag(sync=True)
-    vertexColors = Enum(['NoColors', 'FaceColors', 'VertexColors'], 'NoColors').tag(sync=True)
+    vertexColors = Enum(Colors, 'NoColors').tag(sync=True)
 
 
 class LineDashedMaterial(_LineMaterial):
@@ -567,7 +582,7 @@ class LineDashedMaterial(_LineMaterial):
     scale = CFloat(1.0).tag(sync=True)
     dashSize = CFloat(3.0).tag(sync=True)
     gapSize = CFloat(1.0).tag(sync=True)
-    vertexColors = Enum(['NoColors', 'FaceColors', 'VertexColors'], 'NoColors').tag(sync=True)
+    vertexColors = Enum(Colors, 'NoColors').tag(sync=True)
     fog = Bool().tag(sync=True)
 
 
@@ -576,7 +591,7 @@ class NormalMaterial(Material):
     _model_name = Unicode('NormalMaterialModel').tag(sync=True)
 
     morphTargets = Bool().tag(sync=True)
-    shading = Enum(['SmoothShading', 'FlatShading', 'NoShading'], 'SmoothShading').tag(sync=True)
+    shading = Enum(Shading, 'SmoothShading').tag(sync=True)
     wireframe = Bool().tag(sync=True)
     wireframeLinewidth = CFloat(1.0).tag(sync=True)
 
@@ -603,10 +618,10 @@ class ShaderMaterial(Material):
     lights = Bool().tag(sync=True)
     morphNormals = Bool().tag(sync=True)
     wireframe = Bool().tag(sync=True)
-    vertexColors = Enum(['NoColors', 'FaceColors', 'VertexColors'], 'NoColors').tag(sync=True)
+    vertexColors = Enum(Colors, 'NoColors').tag(sync=True)
     skinning = Bool().tag(sync=True)
     fog = Bool().tag(sync=True)
-    shading = Enum(['SmoothShading', 'FlatShading', 'NoShading'], 'SmoothShading').tag(sync=True)
+    shading = Enum(Shading, 'SmoothShading').tag(sync=True)
     linewidth = CFloat(1.0).tag(sync=True)
     wireframeLinewidth = CFloat(1.0).tag(sync=True)
 
@@ -646,7 +661,7 @@ class Line(Mesh):
     _view_name = Unicode('LineView').tag(sync=True)
     _model_name = Unicode('LineModel').tag(sync=True)
 
-    type = Enum(['LineStrip', 'LinePieces'], 'LineStrip').tag(sync=True)
+    type = Enum(Lines, 'LineStrip').tag(sync=True)
     material = Instance(_LineMaterial).tag(sync=True, **widget_serialization)
 
 
@@ -749,6 +764,8 @@ class Scene(Object3d):
 class Effect(Widget):
     _view_module = Unicode(npm_pkg_name).tag(sync=True)
     _model_module = Unicode(npm_pkg_name).tag(sync=True)
+    _model_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
+    _view_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
 
 
 class AnaglyphEffect(Effect):
@@ -761,10 +778,12 @@ class Renderer(DOMWidget):
     _model_module = Unicode(npm_pkg_name).tag(sync=True)
     _view_name = Unicode('RendererView').tag(sync=True)
     _model_name = Unicode('RendererModel').tag(sync=True)
+    _model_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
+    _view_module_version = Unicode(PYTHREEJS_VERSION).tag(sync=True)
 
     width = Unicode('600').tag(sync=True)  # TODO: stop relying on deprecated DOMWidget attribute
     height = Unicode('400').tag(sync=True)
-    renderer_type = Enum(['webgl', 'canvas', 'auto'], 'auto').tag(sync=True)
+    renderer_type = Enum(Renderers, 'auto').tag(sync=True)
     scene = Instance(Scene).tag(sync=True, **widget_serialization)
     camera = Instance(Camera).tag(sync=True, **widget_serialization)
     controls = List(Instance(Controls)).tag(sync=True, **widget_serialization)
